// ******************************************************
// * CienaAdaptor.js
// *
// * Description:
// *
// * Date          Author          Comments
// * --------------------------------------
// * 08/16/06      Jeff Pratt      Initial version
// * 09/14/06      Jeff Pratt      Added CienaPathTable Repository
// * 11/07/06	   Jeff Pratt	   Added removeSpecialChars
// * 11/10/06      Jeff Pratt      Removed CienaPathTable Repository
// *                               Added cienaFormulaTableArray
// * 02/20/07	   Jeff Pratt	   Updated timestamp from MMSS to HHMM
// * 03/08/07	   Jeff Pratt	   Updated timestamp from HHMM to HH_MM
// * 03/08/07	   Jeff Pratt	   Updated header info to use " : " as a separater
// * 03/15/07	   Jeff Pratt	   Updated header info to use " , " as a separater
// * 03/15/07	   Jeff Pratt	   Removed removeSpecialChars function
// *
// ******************************************************


// ******************************************************
// * DEBUG SETUP
// ******************************************************

var DEBUG = true;

function logMessage(type, logdata) {
   if ( DEBUG ) 
   {
       PV.Logger.logInfo("DEBUG", type, logdata);
   }
}

// ******************************************************
// * SET GLOBAL VARIABLES AND INIT INVENTORY
// ******************************************************

var modelInterface;
var linenumber;
var mydate;
var vendorname;
var emsname;

PV.adaptor.modelInterface = PV.ModelInterface();
modelInterface = PV.adaptor.modelInterface;
modelInterface.initializeForInventory();
PV.StringFieldSchema.prototype.defaultValue = "";
PV.import('CienaFormulaTable');


// ******************************************************
// * ADAPTOR SECTION
// ******************************************************

PV.adaptor.initializeSchemas = function initializeSchemas() {
	initCEMS(this);
};

function initCEMS(context)
{
    logMessage("INIT_CEMS", "Entering");

    context.schemas.add(
	PV.TextInputSchema(
		"CEMS",
		{onOpen: function() { linenumber = 0 },
		onClose: function() { PV.adaptor.modelInterface.commit() }},
		PV.DelimitedRecordSchema(
			"fileName",
			{maxOccurs: 1,
			fieldSeparator: ",",
			recordSeparator: "\r",
			onRead: function(record) { processFileName(record) }},
			PV.StringFieldSchema("Label"),
			PV.StringFieldSchema("FileName")),
		PV.RegExpRecordSchema("supplierId", {maxOccurs: 1, recordSeparator: "\r"}),
		PV.RegExpRecordSchema("emsId", {maxOccurs: 1, recordSeparator: "\r"}),
		PV.RegExpRecordSchema("fileType", {maxOccurs: 1, recordSeparator: "\r"}),
		PV.RegExpRecordSchema("fileCreateTime", {maxOccurs: 1, recordSeparator: "\r"}),
		PV.RegExpRecordSchema("emptyLine", {maxOccurs: 1, recordSeparator: "\r"}),
		PV.RegExpRecordSchema("headerLine", {maxOccurs: 1, recordSeparator: "\r"}),
 		PV.DelimitedRecordSchema(
        		"data",
        		{fieldSeparator: ",",
        		recordSeparator: "\r",
        		maxOccurs: Infinity,
        		hasHeaderRecord: false,
        		onRead: function(record) { processRecord(record,PV.adaptor.modelInterface) }},
			PV.StringFieldSchema("NetworkDeviceID"),
			PV.StringFieldSchema("DeviceSubElementID"),
			PV.StringFieldSchema("DeviceSubElementType"),
			PV.StringFieldSchema("TimeHHMM"),
			PV.StringFieldSchema("LineRate"),
			PV.StringFieldSchema("Laser Temperature"),
			PV.StringFieldSchema("Power Transmitted"),
			PV.StringFieldSchema("Power Received"),
			PV.StringFieldSchema("Laser Bias Current (LBC)"),
			PV.StringFieldSchema("Unavailable Second - Near End Rx"),
			PV.StringFieldSchema("Unavailable Second - Far End Rx"),
			PV.StringFieldSchema("Errored Second - Near End Rx"),
			PV.StringFieldSchema("Errored Second - Far End Rx"),
			PV.StringFieldSchema("Severely Errored Seconds - Near End Rx"),
			PV.StringFieldSchema("Severely Errored Seconds - Far End Rx"),
			PV.StringFieldSchema("Background Block Error (BBE) - Near End Rx"),
			PV.StringFieldSchema("Background Block Error (BBE) - Far End Rx"),
			PV.StringFieldSchema("Transmitted Frames"),
			PV.StringFieldSchema("Received Frames"),
			PV.StringFieldSchema("Errored Frames Rx"),
			PV.StringFieldSchema("Undersized Packets Rx"),
			PV.StringFieldSchema("Octets Tx"),
			PV.StringFieldSchema("Octets Rx"),
			PV.StringFieldSchema("Policing Discards Rx"),
			PV.StringFieldSchema("Oversized Packets Rx"),
			PV.StringFieldSchema("Number of 64 byte/octet frames Rx"),
			PV.StringFieldSchema("Number of frames Rx between 65/127 bytes/octets"),
			PV.StringFieldSchema("Number of frames Rx between 128/255 bytes/octets"),
			PV.StringFieldSchema("Number of frames Rx between 256/511 bytes/octets"),
			PV.StringFieldSchema("Number of frames Rx between 512/1023 bytes/octets"),
			PV.StringFieldSchema("Number of frames Rx between 1024/1518 bytes/octets"),
			PV.StringFieldSchema("Count of Unicast Packets - Tx"),
			PV.StringFieldSchema("Count of Unicast Packets - Rx"),
			PV.StringFieldSchema("Count of Multicast Packets - Tx"),
			PV.StringFieldSchema("Count of Multicast Packets - Rx"),
			PV.StringFieldSchema("Count of Broadcast Packets - Tx"),
			PV.StringFieldSchema("Count of Broadcast Packets - Rx"),
			PV.StringFieldSchema("Jabbers"),
			PV.StringFieldSchema("Discarded Frames"),
			PV.StringFieldSchema("Loss of Signal in Seconds")
		)
	)
    );

    logMessage("INIT_CEMS", "Leaving");

}

function processFileName(record) {
    logMessage("PROCESS_FILE_NAME", "Entering");

    mydate = dateFromFilename(record);
    emsname = nameFromFilename(record);
    vendorname = vendorFromFilename(record);		    

    logMessage("PROCESS_FILE_NAME", "Leaving");
}

function processRecord(record,modelInterface) {
    logMessage("PROCESS_RECORD", "Entering");

    var newElement;
    var newSubelement;

    linenumber = linenumber + 1;

    if (record.NetworkDeviceID != 'EOF' && record.NetworkDeviceID != undefined) {

	newElement = createElement(record,modelInterface);
	newSubelement = createSubelement(record,modelInterface,newElement);
	createMetric(record,modelInterface,newSubelement);
	modelInterface.commit();
    }

    logMessage("PROCESS_RECORD", "Leaving");
}

function createElement(record,modelInterface) {
    logMessage("CREATE_ELEMENT", "Entering");

    var element;
    var elname = record.NetworkDeviceID;

    element = modelInterface.Element();
    element.name = elname;
    element.state = true;
    element.origin = 'Ciena';

    logMessage("CREATE_ELEMENT", element.name);
    logMessage("CREATE_ELEMENT", "Leaving");

    return element;
}

function createSubelement(record,modelInterface,element) {
    logMessage("CREATE_SUBELEMENT", "Entering");

    var subelement;
    var subelname = record.DeviceSubElementID;
    var subeltype = record.DeviceSubElementType;

    logMessage("CREATE_SUBELEMENT", subelname);

    subelement = modelInterface.Subelement();
    subelement.family = subeltype;
    subelement.instance = element.name+"."+subelname;
    subelement.name = element.name+"."+subelname;
    subelement.label = element.name+"."+subelname;
    subelement.origin = 'Ciena';

    subelement = addProperties(record,element,subelement,subelname,subeltype);
    element.addSubelement(subelement);

    logMessage("CREATE_SUBELEMENT", "Leaving");

    return subelement;
}

function addProperties(record,element,subelement,subelname,subeltype) {
    logMessage("CREATE_PROPERTIES", "Entering");

    var newProps = subelname.split("/");
    var lineRate = convertLineRate(record);

    subelement.addProperty('name',element.name+"."+subelname);
    subelement.addProperty('state',true);
    subelement.addProperty('speed',lineRate);
    subelement.addProperty('nmVendor',vendorname);
    subelement.addProperty('DeviceVendor','Ciena');
    subelement.addProperty('nmStatistics',subeltype);
    subelement.addProperty('ems',emsname);
    subelement.addProperty('type',element.name);

    for (var i = 0; i < newProps.length; i++) 
    {
	var newValues = newProps[i].split("=");

	logMessage("CREATE_PROPERTIES", newValues);
	if (newValues.length == 2) {
		subelement.addProperty(newValues[0], newValues[1]);
	}
	else {
		logMessage("CREATE_PROPERTIES", "Unexpected format for properties");
	}
    }

    logMessage("CREATE_PROPERTIES", "Leaving");

    return subelement;
}

function createMetric(record,modelInterface,subelement) {
	logMessage("CREATE_METRIC", "Entering");

	var theMetric;
	var newTime;
	var cleanMetricValue;

	newTime = mydate+record.TimeHHMM.substr(3,2);
	var mytimestamp = createTime(newTime);

	logMessage("CREATE_METRIC", "mytimestamp: "+mytimestamp.toString());	

	for (var name in record)
	{
		switch (name)
		{
		case "NetworkDeviceID":
		case "DeviceSubElementID":
		case "DeviceSubElementType":
		case "TimeHHMM":
		case "LineRate":
			// Skip field
			logMessage("CREATE_METRIC", "Skipping: "+name);
			break;
		default:
			logMessage("CREATE_METRIC", "The metric: "+name);

			theMetric = modelInterface.FloatMetric();
			theMetric.name = cienaFormulaTableArray[name.toString()];
			theMetric.timestamp = mytimestamp;

			logMessage("CREATE_METRIC", theMetric.name);

			if (isNaN(record[name]) || record[name] == undefined || record[name] == "") {
				logMessage("CREATE_METRIC", "NIL value found - SKIPPING METRIC");
			}
			else {
				logMessage("CREATE_METRIC", "A number value "+record[name]);
				theMetric.value = record[name];
				subelement.addMetric(theMetric);
			}

			break;
		}
	}

	logMessage("CREATE_METRIC", "Leaving");
}

function createTime(ts) {
	logMessage("CREATE_TIME", "Entering");

	var year = ts.substr(0,4);
	var month = ts.substr(4,2);
	var day = ts.substr(6,2);
	var hour = ts.substr(8,2);
	var minute = ts.substr(10,2);
	var time = new Date(year,month-1,day,hour,minute);

	logMessage("CREATE_TIME", time);
	logMessage("CREATE_TIME", "Leaving");

	return time
}

function dateFromFilename(record) {
	logMessage("DATE_FROM_FILENAME", "Entering");

	var byDash = record.FileName.split("_");
	var month = byDash[3];
	var day = byDash[2];
	var year = byDash[4];
	var hour = byDash[5].substr(0,2);
	var date = year+month+day+hour;

	logMessage("DATE_FROM_FILENAME", date);
	logMessage("DATE_FROM_FILENAME", "Leaving");

	return date;
}

function nameFromFilename(record) {
	logMessage("NAME_FROM_FILENAME", "Entering");

	var byDash = record.FileName.split("_");

	logMessage("NAME_FROM_FILENAME", byDash[1]);
	logMessage("NAME_FROM_FILENAME", "Leaving");

	return byDash[1];
}

function vendorFromFilename(record) {
	logMessage("VENDOR_FROM_FILENAME", "Entering");

	var byDash = record.FileName.split("_");

	logMessage("VENDOR_FROM_FILENAME", byDash[0]);
	logMessage("VENDOR_FROM_FILENAME", "Leaving");

	return byDash[0];
}

function convertLineRate(record) {
    logMessage("CONVERT_LINERATE", "Entering");

    var lineRateId = record.LineRate;
    logMessage("CONVERT_LINERATE", "Line Rate ID: "+lineRateId);

    var lineRateValue = "UNKNOWN";
    var lineRateArry = {
	1:"OC3_STM1 - 155 Mbps",
    	2:"OC12_STM4 - 622 Mbps",
    	3:"OC48_STM16 - 2.5 Gbps",
    	4:"OC192_STM64 - 9.6 Gbps",
    	5:"GIGE - 1000 Mbps",
    	6:"10GIGE - 10 Gbps",
    	7:"OTU1 - 2.7 Gbps",
    	8:"OTU2 - 10.7 Gbps",
    	9:"WDM - 40+ Gbps"
    };

    if (lineRateId > 0 && lineRateId < 10) {
	lineRateValue = lineRateArry[lineRateId];
    }
    else {
    	logMessage("CONVERT_LINERATE", "Line Rate Id Invalid: "+lineRateId);
    }
    logMessage("CONVERT_LINERATE", "Line Rate: "+lineRateValue);

    logMessage("CONVERT_LINERATE", "Leaving");

    return lineRateValue;
}
